@@ -7,11 +7,24 @@
 ************************************************************/
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include "mpeg.hh"
 #include "mpegOut.hh"
 #include "chunkTab.hh"
 #include "id3command.hh"
 
+// philipp
+bool desperate_mode = false;
+bool preserve_header = false;
+bool print_progress = true;
+// [L] ifdef the endMessage
+#ifdef NEED_END_MESSAGE
+void endMessage(void);
+#endif
+
+bool aspect_correction  = false;
+byte forced_sequence_header = 0;
+
 char* progname;
 #ifndef NOSIGNAL_H
 // for bug report
@@ -24,6 +37,7 @@ int myargc;
 c_char mpgtx_stack[mpgtx_stack_count];
 int mpgtx_stack_current=0;
 
+
 void CatchBug(int signum)
 {
 	//print a formated bug report and warn user
@@ -190,7 +204,16 @@ void print_help(programname invocation){
 		printf("  -#              where # is a number, split the following file in #.\n");
 	if (invocation != mpgcat){
 		printf("Options  :\n");
+		printf("  -N              Do not print out excessive information while processing\n");
+		printf("                  If you use MacOS Classic, this is RECOMMENDED\n");
+		printf("                  Otherwise mpgtx will be dead slow!\n");
 		printf("  -f              force overwriting of files, don't ask for confirmation\n");
+		printf("  -P              keep headers of mpegs which sometimes is usefull\n");
+		printf("  -X              Try to search for a valid PACK header in the whole file\n");
+		printf("                  This may result in an endless loop if not successfull!\n");
+		printf("  -An             While joining or splitting, aspect ratio will be forced to:\n");
+		printf("                  n=1 -> 1:1, n=2 -> 4:3, n=3 -> 16:9, n=4 -> 2.21:1 \n");
+		printf("                  All Numbers from 0 - 16 will be accepted but the above are valid!\n"); 
 		if (invocation != mpgjoin)
 			printf("  -b NAME         set the basename for the output files\n");
 		if (invocation != mpgsplit && invocation != mpgdemux){
@@ -280,6 +303,7 @@ int main (int argc,char** argv){
 	argtype lastarg=none;
 	programname invocation;
 	bool demux=false;
+	int tmp_seq;
 
 
 
@@ -388,6 +412,27 @@ int main (int argc,char** argv){
 						case 'd' : 
 							demux=true; 
 							break;
+						case 'X' :
+							desperate_mode =true;
+							break;
+						case 'P' :
+							preserve_header = true;
+							break;
+						case 'N':
+							print_progress = false;
+							break;
+						case 'A':
+							aspect_correction = true;
+							tmp_seq = -1;
+							sscanf(&(argv[i][2]),"%d",&tmp_seq);
+							if ((tmp_seq < 0 ) || (tmp_seq > 15))
+							{
+ 								fprintf(stderr,"invalid Value for aspect correction %d, exit!\n",tmp_seq);
+								exit(1);
+							}
+							forced_sequence_header = tmp_seq << 4;
+							break;
+
 						case 'h' : print_help(invocation);
 									  return 0;
 									  break;
@@ -459,6 +504,7 @@ int main (int argc,char** argv){
 						case 'v':
 									  printf("%s Version %s\n",progname,MPGTX_VERSION);
 									  printf("Copyleft 2001 Laurent Alacoque <laureck@users.sourceforge.net>\n");
+									  printf("Partial work 2002 by Philipp Biermann <mpegcut@biermann.org>\n");
 									  printf("updates, bugs, patch, money :    http://mpgtx.sourceforge.net\n");
 									  return 0;
 									  break;
@@ -543,6 +589,9 @@ int main (int argc,char** argv){
 
 	if(want_info){
 		Tab.PrintInfos();
+#ifdef NEED_END_MESSAGE
+		endMessage();
+#endif
 		return 0;
 	}	
 
@@ -566,9 +615,12 @@ int main (int argc,char** argv){
 			fprintf(stderr,"No base name specified, defaulting to basename : chunk\n");
 			mybasename="chunk";
 		}
-		demuxer Dmux(ChunkTab[0]->mpegfile, mybasename);
+		demuxer Dmux(ChunkTab[0]->mpegfile, mybasename,confirm_files);
 		Dmux.Process();
 		//check the errors!
+#ifdef NEED_END_MESSAGE
+		endMessage();
+#endif
 		return 0;
 	}
 	// from here we need at least one chunk
@@ -608,7 +660,7 @@ int main (int argc,char** argv){
 		}
 		if (!strcmp(myoutfile,"-")){
 			//output to stdout
-			current_out=TheFac.NewMpegFrom(ChunkTab[0]->mpegfile,stdout,true);
+			current_out=TheFac.NewMpegFrom(ChunkTab[0]->mpegfile,stdout);
 			if (!current_out){
 				fprintf(stderr,"\nFatal : could not process file\n");
 			}
@@ -649,6 +701,9 @@ int main (int argc,char** argv){
 				fprintf(stderr,"\n");
 		}
 		current_out->Finish();
+#ifdef NEED_END_MESSAGE
+		endMessage();
+#endif
 		return 0;
 	}
 
@@ -723,7 +778,7 @@ int main (int argc,char** argv){
 
 			}
 
-			current_out = TheFac.NewMpegFrom(ChunkTab[0]->mpegfile, myoutfile, true);
+			current_out = TheFac.NewMpegFrom(ChunkTab[0]->mpegfile, myoutfile);
 			if (!current_out) {
 				fprintf(stderr,"\nFatal : could not process %s\n", myoutfile);
 				return 1;
@@ -739,7 +794,9 @@ int main (int argc,char** argv){
 			current_out->Finish();
 		}
 		delete [] myoutfile;
-
+#ifdef NEED_END_MESSAGE
+		endMessage();
+#endif
 		return 0;
 	}
 
@@ -750,4 +807,8 @@ int main (int argc,char** argv){
 }
 
 
-
+#ifdef NEED_END_MESSAGE
+void endMessage(void) {
+		fprintf(stderr,"-------Done--------\n");
+}
+#endif
