@@ -7,7 +7,14 @@
  ************************************************************/
 
 #include "mpeg.hh"
+#include <stdlib.h>
+//philipp 
+#include <stdio.h>
+extern bool desperate_mode;
+extern bool preserve_header;
+int mpeg2found = 0;
 
+// #define _DEBUG_
 int mpeg_bitrate_index [2][3][16] =
 {{{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0},
 	{0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0},
@@ -308,8 +315,10 @@ mpeg::~mpeg() {
 
 // map a buffer onto file for efficiency
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 byte mpeg::GetByte(off_t offset) {
 	BTRACK;
@@ -326,6 +335,7 @@ byte mpeg::GetByte(off_t offset) {
 
 			RTRACK;
 			ATRACK("GetByte Invalid offset");
+			fprintf(stderr,"mpgtx: AT EOF - please stop me!\n");
 			return 0x11; 
 		}
 		nread = fread(buffer, 1, BUFFERSIZE, MpegFile);
@@ -340,6 +350,7 @@ byte mpeg::GetByte(off_t offset) {
 						offset, FileName, FileSize);
 			RTRACK;
 			ATRACK("GetByte Invalid offset");
+			fprintf(stderr,"mpgtx: AT EOF - please stop me!\n");
 			return 0x11; 
 		}
 	}
@@ -356,8 +367,10 @@ byte mpeg::Byte(off_t offset){
 
 // same as above but improved for backward search
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 byte mpeg::bdGetByte(off_t offset){
 	BTRACK;
@@ -392,8 +405,10 @@ byte mpeg::bdGetByte(off_t offset){
 
 // make sure the sequence 0x00 00 01 mark
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 bool mpeg::EnsureMPEG(off_t offset, marker mark){
 	BTRACK;
@@ -411,8 +426,10 @@ bool mpeg::EnsureMPEG(off_t offset, marker mark){
 
 // get a two byte size
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 
 unsigned short int mpeg::GetSize(off_t offset ) {
@@ -789,9 +806,10 @@ bool mpeg::ParseVideo(off_t myoffset){
 	// field.
 	Video->video_header_size = header_end - header_start;
 	Video->video_header = new byte[Video->video_header_size];
-
+	// fprintf(stderr,"philipp found header: 0x%qx 0x%qx\n", header_end, header_start);
 
 	FSEEK(MpegFile, header_start, SEEK_SET);
+
 	if(fread(Video->video_header, Video->video_header_size, 1, MpegFile) != 1) {
 		// couldn't read video header
 		if (Verboseness == mpeg_VERBOSE)
@@ -821,6 +839,8 @@ bool mpeg::ParseSystem() {
 	off_t PACKlength = 0;
 	unsigned short packetsize;
 	byte packettype;
+	int found = 0;
+
 
 	// the MPEG system file should start with 
 	// the video start sequence, if not exit.
@@ -838,10 +858,53 @@ bool mpeg::ParseSystem() {
 	
 	if (!EnsureMPEG(offset, 0xBA)){
 		//this file does not begin with a pack!
-		fprintf(stderr,"mmm, this file does not start with a pack\n");
+		fprintf(stderr,"mmm, this file does not start with a pack, offset: " _OFF_d " \n",offset);
+		fprintf(stderr,"use the desperate_mode switch as the first option -X to search for a header in the whole file!\n");
+		fprintf(stderr,"if you want to force the operation. May yield to an endless loop if no valid header is found!\n");
 		if (GetByte(offset+2) != 0x01) {
+			fprintf(stderr,"Does not even begin with a 00 00 01 xx sequence!\n");
 			// does not even begin with a 00 00 01 xx sequence !
-			return false;
+			if (desperate_mode)
+			{
+// philipp tries here to search further ....
+			offset +=3;
+			while (found  == 0) {
+				fprintf (stderr,"Searching for a mpeg signature:\n");
+				fprintf(stderr,"Searching for 0x00 at offset:           ");
+				while(GetByte(offset) != 0x00) {
+					fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
+					offset++;
+				}
+				fprintf(stderr,"\nSearching for != 0x00 at offset:           ");
+				while(GetByte(offset) == 0x00) {
+					offset++;
+					fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
+				}
+				// now from the beginning ..
+				offset -=2;
+				if (!EnsureMPEG(offset, 0xBA)){
+					fprintf(stderr,"\nno success......\n");
+					offset += 3;
+				}
+				else {
+					fprintf(stderr,"\nsuccess! at " _OFF_d " \n",offset);
+					found = 1;
+					fprintf(stderr,"I dont know which kind of crazy header you have put into this file!\n");
+					if (!preserve_header) {
+					fprintf(stderr,"But I will assume this is ok. If you split or join this file, it will be discarded\n");
+					}
+					else
+					{
+					fprintf(stderr,"But I will assume this is ok. And if you split this file, it will be inserted\n");
+					}
+					fprintf(stderr,"Before any segment. This movie maybe wil not play with quicktime, use Videolan instead\n");
+					fprintf(stderr,"If you dont want this, please correct the file, so its start with a valid Pack!\n");
+
+				}
+			}
+			}
+			if (found == 0)
+				return false;
 		}
 	}
 
@@ -854,11 +917,14 @@ bool mpeg::ParseSystem() {
 	// tries to find the two system packets and store them
 	bool keep_going = true;
 	marker mark = 0;
-//	fprintf(stderr, "Searching for system packets (audio, video).\n");
+	// fprintf(stderr, "Philipp: Searching for system packets (audio, video).\n");
 
 	while (keep_going)
 	{
+		// fprintf(stderr,"philipp: offset4 ?: 0x%qx\n",offset);
+
 		offset = FindNextMarker(offset, &mark);
+		// fprintf(stderr,"philipp: offset5 ?: 0x%qx\n",offset);
 
 		// if error exit
 		if (offset == -1) {
@@ -897,8 +963,11 @@ bool mpeg::ParseSystem() {
 		//     March 27 changed because of variable length PACK in mpeg 2
 
 		// are we lost?
+		// fprintf(stderr,"philipp: offset3 ?: 0x%qx\n",offset);
+
 		off_t startofpack = bdFindNextMarker(offset, 0xBA);
-		
+		// fprintf(stderr,"philipp: offset2 ?: 0x%qx\n",offset);
+
 		if (startofpack !=-1){
 			//found a PACK before system packet, compute its size (mpeg1 != mpeg2)
 			if ((GetByte(startofpack + 4) & 0xF0) == 0x20) {
@@ -982,11 +1051,24 @@ bool mpeg::ParseSystem() {
 				new byte [System->video_system_header_length];
 
 			// keep track of the initial timestamp
-			if (PACKlength == 12)
+			if (PACKlength == 12) {
 				System->initial_TS = ReadTS(offset - PACKlength);
-			else
-				System->initial_TS = ReadTSMpeg2(offset - PACKlength);			
-
+				mpeg2found = 0;
+			}
+			else {
+				// off_t firstmpeg2gop = FindNextMarker(1, 0xB8);
+#ifdef _DEBUG_
+				fprintf(stderr,"Now read mpeg2 initial_TS at [" _OFF_x "] \n",offset - PACKlength);
+				if (getchar() == 'y')
+				 	print_all_ts(0xba);
+				while(getchar() != '\n')
+					;
+#endif
+				mpeg2found = 1;
+//				System->initial_TS = ReadTSMpeg2(firstmpeg2gop + 4);
+				System->initial_TS = ReadTSMpeg2(offset - PACKlength);
+				 
+			}
 			FSEEK(MpegFile, (offset - PACKlength), SEEK_SET);
 
 			fread(
@@ -1023,6 +1105,7 @@ bool mpeg::ParseSystem() {
 
 	// Video!
 	offset = FindNextMarker(0, 0xB3);
+	// fprintf(stderr,"philipp: offset1 ?: 0x%qx\n",offset);
 	if (offset == -1) {
 		//couldn't find any valid video
 		if (Verboseness == mpeg_VERBOSE)
@@ -1085,11 +1168,11 @@ bool mpeg::ParseSystem() {
 	System->first_video_packet = new byte[xB3_offset-offset];
 	System->first_video_packet_length = xB3_offset-offset;
 
-
+	readHeader(MpegFile,offset,1); // save this if needed for later
 	FSEEK(MpegFile,offset,SEEK_SET);
-
+	
 	fread(System->first_video_packet, 1, xB3_offset-offset, MpegFile);
-
+	
 
 	// Now look for an audio packet if necessary
 	HasAudio = false;
@@ -1126,9 +1209,18 @@ bool mpeg::ParseSystem() {
 		//standard mpeg1 pack
 		duration = ReadTS(last_pack + 4);		
 	} else {
-		if((GetByte(last_pack + 4) & 0xC0) == 0x40) {
+		last_pack = bdFindNextMarker(FileSize - 8, 0xB8);
+		if(((GetByte(last_pack + 4) & 0xC0) == 0x40) || (mpeg2found == 1)) {
 			//new mpeg2 pack : 14 bytes + stuffing
+#ifdef _DEBUG_
+			fprintf(stderr,"now getting mpeg2 running time");
+#endif
+			last_pack = bdFindNextMarker(FileSize - 8, 0xBA);
 			duration = ReadTSMpeg2(last_pack + 4);
+#ifdef _DEBUG_
+			fprintf(stderr," : %lf\n", duration);
+#endif
+
 		} else {
 			//wazup?
 			duration = ReadTS(last_pack + 4);
@@ -1444,8 +1536,10 @@ void mpeg::SecsToHMS(char* HMS, float duration){
 
 // find next 0x 00 00 01 xx sequence, returns offset or -1 on err
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 off_t mpeg::FindNextMarker(off_t from){
 	BTRACK;
@@ -1473,8 +1567,10 @@ off_t mpeg::FindNextMarker(off_t from){
 // find next 0x 00 00 01 xx sequence, returns offset or -1 on err and
 // change mark to xx
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 off_t mpeg::FindNextMarker(off_t from, marker* mark){
 	BTRACK;
@@ -1499,8 +1595,11 @@ off_t mpeg::FindNextMarker(off_t from, marker* mark){
 
 // find next 0X00 00 01 mark
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
+
 
 off_t mpeg::FindNextMarker(off_t from, marker mark){
 	BTRACK;
@@ -1531,8 +1630,10 @@ off_t mpeg::FindNextMarker(off_t from, marker mark){
 
 // same as above but optimized for backward search
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 off_t mpeg::bdFindNextMarker(off_t from, marker mark) {
 	BTRACK;
@@ -1554,8 +1655,10 @@ off_t mpeg::bdFindNextMarker(off_t from, marker mark) {
 }
 
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 off_t mpeg::bdFindNextMarker(off_t from, marker* mark) { 
 	BTRACK;
@@ -1581,8 +1684,10 @@ off_t mpeg::bdFindNextMarker(off_t from, marker* mark) {
 //       look at what we have to do to get a timestamp :/
 
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 double mpeg::ReadTS(off_t offset) {
 	BTRACK;
@@ -1609,8 +1714,10 @@ double mpeg::ReadTS(off_t offset) {
 }
 
 #ifndef _WIN32
+#ifndef _MACOSX
 inline
 #endif
+#endif
 
 double mpeg::ReadTSMpeg2(off_t offset){
 	BTRACK;
@@ -1618,16 +1725,19 @@ double mpeg::ReadTSMpeg2(off_t offset){
 	unsigned long low4Bytes;
 	unsigned long sys_clock_ref;
 	double TS;
+#ifdef _DEBUG_
+	// fprintf(stderr,"Offset: [" _OFF_x "] The bytes...: %2x %2x %2x %2x %2x %2x",offset,GetByte(offset + 0),GetByte(offset + 1),GetByte(offset + 2),GetByte(offset + 3),GetByte(offset + 4),GetByte(offset + 5) );
+#endif
 
 	highbit = (GetByte(offset)&0x20)>>5;
 
-	low4Bytes= 		((GetByte(offset		) & 0x18)	>> 3)	<< 30;
-	low4Bytes |= 	 (GetByte(offset		) & 0x03)	<< 28;
-	low4Bytes |=	  GetByte(offset + 1	)				<< 20;
-	low4Bytes |= 	((GetByte(offset + 2	) & 0xF8)	>> 1)	<< 15;
-	low4Bytes |=	 (GetByte(offset + 2	) & 0x03)	<< 13;
-	low4Bytes |=	  GetByte(offset + 3	)				<< 5;
-	low4Bytes |=	 (GetByte(offset + 4	))				>> 3;
+	low4Bytes= 		((GetByte(offset) & 0x18) >> 3)	<< 30;
+	low4Bytes |= 	 (GetByte(offset) & 0x03) << 28;
+	low4Bytes |=	  GetByte(offset + 1	) << 20;
+	low4Bytes |= 	 (GetByte(offset + 2	) & 0xF8) << 12;
+	low4Bytes |=	 (GetByte(offset + 2	) & 0x03) << 13;
+	low4Bytes |=	  GetByte(offset + 3	) << 5;
+	low4Bytes |=	 (GetByte(offset + 4	)) >> 3;
 
 	sys_clock_ref=(GetByte(offset + 4) & 0x3) << 7;
 	sys_clock_ref|=(GetByte(offset + 5) >> 1);
@@ -1639,11 +1749,15 @@ double mpeg::ReadTSMpeg2(off_t offset){
 		// this is what I understood... CHECK
 		// if not zero, then we have a 27 MHz accuracy with a max of 27 MHz
 		// so clockfreq might well be 27MHz / sys_clock_ref
-		TS /= (double)(27000000 / sys_clock_ref);
-	}
+		//  nonsense: TS /= (double)(27000000 / sys_clock_ref);
+		TS /= (double)(STD_SYSTEM_CLOCK_FREQ);
+	} 
+
+
+
 	//debug
 #ifdef _DEBUG_
-	printf("read TS2 : base : 0x%d,%032lx ref : %016lx\n",highbit,low4Bytes,sys_clock_ref);
+	// printf("read TS2 : %lf, base : 0x%d,%032lx ref : %016lx\n",TS,highbit,low4Bytes,sys_clock_ref);
 #endif
 	return TS;
 }
@@ -2621,3 +2735,60 @@ void mpeg_descriptors::PrintInfos(char* prefix){
 	delete[] description;
 }
 
+header_buf *readHeader(FILE *myMpegfile, off_t offset, int rw) 
+{
+
+static byte *nix;
+static off_t size;
+static header_buf *p;
+
+	if (rw == 1)
+	{
+		p = new header_buf;
+		size = offset;
+		nix = new byte[size];
+		if (!nix)
+		{
+			fprintf(stderr,"unable to alloc buffer for header\n");
+			exit(1);
+		}
+	        FSEEK(myMpegfile,0L,SEEK_SET);		
+		fread(nix,size,1L,myMpegfile);
+		p->size = size;
+		p->buf = nix;
+		if (preserve_header)
+			fprintf(stderr,"Size of Fix: 0x%qx\n",size);
+	return(NULL);
+	}
+	if (rw == 2)
+	{
+		return(p);
+	}
+
+fprintf(stderr,"unable to understand command\n");
+exit(1);
+}
+
+
+
+
+void mpeg::print_all_ts(byte kind){
+
+off_t p = 0;
+byte *nix;
+double tsx;
+nix = &kind;
+
+while (p != -1)
+{
+	p = FindNextMarker(p,kind);
+	tsx = ReadTSMpeg2(p +4);
+	if (p == -1)
+		break;
+	fprintf (stderr,"offset:  " _OFF_x " TS: %lf\n",p,tsx);
+	p+=4;
+
+}
+
+
+}
\ No newline at end of file
